/* Autoconf patching by David Hedbor, neotron@lysator.liu.se */
/*********************************************************************/
/* file: net.c - do all the net stuff                                */
/*                             TINTIN III                            */
/*          (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t             */
/*                     coded by peter unold 1992                     */
/*********************************************************************/

/*********************************************************************/
/* This is new code to support the repeating connector.              */
/* repeat_connect_mud() and exit_stage_left() were added.     -Snarf */
/*********************************************************************/
/* External stuff needed for repeat connect. */
extern void term_echo();
extern void term_noecho();
extern void myquitsig();	/* Must not be static in main.c. */
extern int is_split;
extern int split_line;
/* Variable global to this file. */
static int connect_interrupted=0;
/* Signal stuff needed for catching SIGINT. */
#include <signal.h>
#ifndef BADSIG
#define BADSIG (void (*)())-1
#endif

#include <ctype.h>
#ifdef HAVE_STRING_H
#include <string.h>
#else
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#endif
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "tintin.h"

/* NOTE!  Some systems might require a #include <net/errno.h>,
 * try adding this if you are really stuck and net.c won't compile.
 * Thanks to Brian Ebersole [Harm@GrimneMUD] for this suggestion.
 */

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#include <arpa/inet.h>
#endif


void do_telnet_protecol();

extern int sessionsstarted, ticker_interrupted;
extern struct listnode *common_aliases, *common_actions, *common_subs;
extern struct session *sessionlist, *activesession;
extern int errno;
/**************************************************/
/* try connect to the mud specified by the args   */
/* return fd on success / 0 on failure            */
/**************************************************/
int connect_mud(host, port, ses)
     char *host;
     char *port;
     struct session *ses;
{
  int sock, connectresult;
  struct sockaddr_in sockaddr;

  if(isdigit(*host))		/* interprete host part */
    sockaddr.sin_addr.s_addr=inet_addr(host);
  else {
    struct hostent *hp;
    if((hp=gethostbyname(host))==NULL) {
      tintin_puts("#ERROR - UNKNOWN HOST.", ses);
      prompt(NULL); 
      return 0;
    }
    memcpy((char *)&sockaddr.sin_addr, hp->h_addr, sizeof(sockaddr.sin_addr));
  }

  if(isdigit(*port))  
    sockaddr.sin_port=htons(atoi(port)); /* inteprete port part */
  else {
    tintin_puts("#THE PORT SHOULD BE A NUMBER.", ses);
    prompt(NULL);
    return 0;
  }

  if((sock=socket(AF_INET, SOCK_STREAM, 0))<0) 
    syserr("socket");

  sockaddr.sin_family=AF_INET;


  tintin_puts("#Trying to connect..", ses);

  alarm(TIMEOUT);	/* configurable in tintin.h */
  ticker_interrupted=FALSE;
  connectresult=connect(sock, (struct sockaddr *)&sockaddr, sizeof(sockaddr));
#if defined(DEBUG)
  alarm(0);
#else
  alarm(1); 
#endif

  if(connectresult || ticker_interrupted) {
    close(sock);
    if(ticker_interrupted)
      tintin_puts("#CONNECTION TIMED OUT.", ses);
    else
      switch(errno) {
       case ECONNREFUSED:
	tintin_puts("#ERROR - CONNECTION REFUSED.", ses);
	break;
       case ENETUNREACH:
	tintin_puts("#ERROR - THE NETWORK IS NOT REACHABLE FROM THIS HOST.", ses);
	break;
       default:
	tintin_puts("#Couldn't connect", ses);
      } 
    prompt(NULL);
    return 0;
  }
  return sock;
}

/*******************************************************/
/* try connect to the mud specified by the args        */
/* return fd on success / prompt for repeat on failure */
/*******************************************************/
int repeat_connect_mud(host, port, ses)
	char			*host;
	char			*port;
	struct session		*ses;
{
	int			sock, connectresult;
	int			connect_attempt=1, time_to_sleep;
	struct sockaddr_in	sockaddr;
	char			connect_response[BUFFER_SIZE], linebuf[80];
	extern void		exit_stage_left();

	connect_interrupted=0;
	sockaddr.sin_family=AF_INET;

	/* Interpret the host. */
	if (isdigit(*host))
		sockaddr.sin_addr.s_addr=inet_addr(host);
	else {
		struct hostent *hp;
		if ((hp=gethostbyname(host))==NULL) {
			tintin_puts("#ERROR - UNKNOWN HOST.", ses);
			prompt(NULL);
			return 0;
		}
		memcpy((char *)&sockaddr.sin_addr, hp->h_addr, sizeof(sockaddr.sin_addr));
	}

	/* Interpret the port. */
	if (isdigit(*port))
		sockaddr.sin_port=htons(atoi(port));
	else {
		tintin_puts("#THE PORT SHOULD BE A NUMBER.", ses);
		prompt(NULL);
		return 0;
	}

	for (;;) {
		sprintf(linebuf, "#Beginning connection attempt %d...", connect_attempt);
		tintin_puts2_nocr(linebuf, ses);
		if ((sock=socket(AF_INET, SOCK_STREAM, 0))<0) syserr("socket");
		alarm(0);
		ticker_interrupted=FALSE;
		alarm(TIMEOUT);	/* configurable in tintin.h */
		connectresult=connect(sock, (struct sockaddr *)&sockaddr, sizeof(sockaddr));
		alarm(0);

		if (connect_interrupted) {
			close(sock);
			#if defined(DEBUG)
				alarm(0);
			#else
				alarm(1);
			#endif
			prompt(NULL);
			return 0;
		}
		else if (connectresult || ticker_interrupted) {
			close(sock);
			if (ticker_interrupted) {
				tintin_puts2("connection timed out.", ses);
				time_to_sleep=WAIT_TIMEOUT;
			}
			else {
				switch(errno) {
				case ECONNREFUSED:
					tintin_puts2("connection refused.", ses);
					time_to_sleep=WAIT_REFUSED;
					break;
				case ENETUNREACH:
					tintin_puts2("network unreachable.", ses);
					time_to_sleep=WAIT_UNREACHABLE;
					break;
				default:
					tintin_puts2("unknown error, aborting.", ses);
					if (signal(SIGINT, myquitsig)==BADSIG)
						syserr("signal SIGINT");
					#if defined(DEBUG)
						alarm(0);
					#else
						alarm(1);
					#endif
					prompt(NULL);
					return 0;
				} 
			}
			if (connect_attempt==1) {
				tintin_puts2_nocr("#Begin auto-retry? [y/n] ", ses);
				fflush(stdin);
				term_echo();
				fgets(connect_response, sizeof(connect_response), stdin);
				term_noecho();
				if (is_split) {
					tintin_puts2_nocr(connect_response, ses);
					/* clear the input line */
					sprintf(linebuf, "[%d;1f[0J", split_line+1);
					write(1, linebuf, strlen(linebuf));
				}
				connect_response[1]='\0';
				if (strcmp(connect_response,"y") && strcmp(connect_response,"Y")) {
					tintin_puts2("#Ok, aborting.", ses);
					#if defined(DEBUG)
						alarm(0);
					#else
						alarm(1);
					#endif
					prompt(NULL);
					return 0;
				}
				else {
					tintin_puts2("#Ok, auto-retry engaged! (control-c to exit)", ses);
					if (signal(SIGINT, exit_stage_left)==BADSIG)
						syserr("signal SIGINT");
				}
			}
			connect_attempt++;

			/* sleep for time_to_sleep seconds */
			/* since sleep() + alarm() is buggy, use this loop instead */
			alarm(0);
			ticker_interrupted=FALSE;
			alarm(1);
			while (time_to_sleep) {
				(void)pause();
				if (ticker_interrupted) {
					/* keep ticking in DEBUG mode */
					#if defined(DEBUG)
						alarm(1);
					#endif
					time_to_sleep--;
					ticker_interrupted=FALSE;
				}
				if (connect_interrupted) {
					#if defined(DEBUG)
						alarm(0);
					#else
						alarm(1);
					#endif
					prompt(NULL);
					return 0;
				}
			}
		}
		else {
			tintin_puts2("connecting!", ses);
			if (signal(SIGINT, myquitsig)==BADSIG)
				syserr("signal SIGINT");
			#if defined(DEBUG)
				alarm(0);
			#else
				alarm(1);
			#endif
			#ifdef CONNECT_BELL
				if (connect_attempt > 1)
					bell_command(ses);
			#endif
			return sock;
		}
	}
}

/*************************************************************/
/* deal with an interrupt signal during repeat_connect_mud() */
/*************************************************************/
void exit_stage_left()
{
	if (signal(SIGINT, SIG_IGN)==BADSIG)
		syserr("signal SIGINT");
	connect_interrupted=1;
	tintin_puts2("#Caught interrupt, exiting.", NULL);
	if (signal(SIGINT, myquitsig)==BADSIG)
		syserr("signal SIGINT");
}

/************************************************************/
/* write line to the mud ses is connected to - add \n first */
/************************************************************/
void write_line_mud(line, ses)
     char *line;
     struct session *ses;
{
  char outtext[BUFFER_SIZE+2];

  strcpy(outtext, line);
  strcat(outtext, "\n\r");
    
  if(write(ses->socket, outtext, strlen(outtext))== -1)
    syserr("write in write_to_mud");
}


/*******************************************************************/
/* read at most BUFFER_SIZE chars from mud - parse protecol stuff  */
/*******************************************************************/
int read_buffer_mud(buffer, ses)
     char *buffer;
     struct session *ses;
{
  int i, didget;
  char tmpbuf[BUFFER_SIZE], *cpsource, *cpdest;
  didget=read(ses->socket, tmpbuf, 512);
  ses->old_more_coming=ses->more_coming;
  if (didget==512) 
    ses->more_coming=1;
  else 
    ses->more_coming=0;
  if(didget<0) 
    return 0; /*syserr("read from socket");  we do this here instead - dunno quite 
                why, but i got some mysterious connection read by peer on some hps */
  
  else if(didget==0) 
    return 0;

  else {
    cpsource=tmpbuf;
    cpdest=buffer;
    i=didget;
    while(i>0) {
      if(*(unsigned char *)cpsource==255) {
        do_telnet_protecol(*cpsource, *(cpsource+1), *(cpsource+2), ses);
        i-=3;
        cpsource+=3;
      }
      else {
        *cpdest++= *cpsource++;
        i--;
      }
    }
  }
  *cpdest='\0';
  return didget;
}


/*****************************************************************/
/* respond according to the telnet protecol - weeeeelllllll..... */
/*****************************************************************/
void do_telnet_protecol(dat0, dat1, dat2, ses)
     int dat0;
     int dat1;
     int dat2;
     struct session *ses;
{
/* we don't do anything here.. why should we? add the stuff yourself if
   you feel like being nice..... */
}
